##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################

# Not only that, but this software is NOT supported by Digital Creations.
# Send comments and questions and comments to jatwood@bluewildefire.com

__doc__ = "ZDiscussions Base Classes"
__version__ = "0.7.5"

import Globals, OFS, Products, AccessControl
from Products.ZCatalog import ZCatalog
from Acquisition import Implicit, aq_parent, aq_base 
from Globals import HTMLFile, Persistent
from DateTime import DateTime

def manage_addZDTopic(self, id, title, REQUEST=None):
    """Add a ZDTopic object"""

    topic = ZDTopic(id, title)
    self._setObject(id, topic)
    if REQUEST is not None:
	return self.manage_main(self, REQUEST)

class ZDItem(OFS.SimpleItem.SimpleItem,
	     OFS.PropertyManager.PropertyManager,
	     Persistent,
	     Implicit,
	     AccessControl.Role.RoleManager):
    """ZDItem object"""

    meta_type = "Z Discussions Item Base Class"

    _properties = (
	{'id':'title', 'type':'string', 'mode':'w'},
	{'id':'author', 'type':'string', 'mode':'w'},
	{'id':'email', 'type':'string', 'mode':'w'},
	{'id':'body', 'type':'text', 'mode':'w'},
	{'id':'in_reply_to', 'type':'string', 'mode':''}
	)

    manage_options = (
	{'label':'Properties', 'action':'manage_propertiesForm'}
	)

    __ac_permissions__ = (
	('Manage properties', ('manage_addProperty',
			       'manage_editProperties',
			       'manage_delProperties',
			       'manage_changeProperties')),
	)

    def __init__(self, id, title, body, author, email, in_reply_to):
	self.id = id
	self.title = title
	self.author = author
	self.email = email
	self.body = body
	self.in_reply_to = in_reply_to
	self.created = self.modified = DateTime()

    def thread_top(self):
        """ returns the top ancestor """
        if len(self.ancestors()) > 0:
                return self.ancestors()[-1]
        else:
                return self

    def ancestors(self):
	"""Return a list of ancestral posts"""
	ancestors = []
	irp = self.in_reply_to
	while irp:
	    parent = getattr(self.aq_parent, irp)
	    ancestors = ancestors + [parent]
	    irp = parent.in_reply_to
	return ancestors

    def children(self, REQUEST=None):
	"""Return a list of replies to this item"""
	if REQUEST is None:
	    REQUEST = self.REQUEST
	children = []
	results = self.searchResults(in_reply_to=self.id) #, sort_on="created")
	children = map(lambda x, self=self, REQUEST=REQUEST:
		        self.getobject(x.data_record_id_, REQUEST),
		        results)
	return children

    def manage_addReply(self, id, title, body, author, email, REQUEST=None):
	"""Add a ZDItem to this ZDTopic"""
    
	item = ZDItem(id, title, body, author, email, self.id)
	self.aq_parent._setObject(id, item)
	if REQUEST is not None:
	    return self.manage_main(self, REQUEST)

    def manage_afterAdd(self, item, container):
	OFS.SimpleItem.SimpleItem.manage_afterAdd(self, item, container)
	self.catalog_object(self, self.absolute_url(1))

    # Am thinkink this is bad -
    # I seem to recall 'self' is not part of the acquisition hierarchy when
    # afterClone is called, resulting in error.  Test.

#    def manage_afterClone(self, item, container):
#        OFS.SimpleItem.SimpleItem.manage_afterClone(self, item, container)
#        self.catalog_object(self, self.absolute_url(1))

    def manage_beforeDelete(self, item, container):
	# Prevent orphans
	for child in self.children():
	    child.in_reply_to = self.in_reply_to
	OFS.SimpleItem.SimpleItem.manage_beforeDelete(self, item, container)
	try:
	    self.uncatalog_object(self.absolute_url(1))
	except ValueError:
	    pass

    def childMessageCount(self):
        """ Counts the number of chilren of this message, including this message"""
        count=1 # this message
        for message in self.children():
                count = count + message.childMessageCount()
        return count

    def threadMessageCount(self):
        """ Looks at the topmost ancestor, and then counts children"""

        message_count = self.thread_top().childMessageCount()
        return message_count

    def childNewestMessageDate(self):
        """ Looks through children and returns the newest message date"""
        newest=self.created
        for message in self.children():
                test_date = message.childNewestMessageDate()
                if test_date > newest:
                       newest = test_date
        return newest

    def threadNewestMessageDate(self):
        """ Looks at the topmost ancestor, and then recurses
            through the messages finding the last created message"""
        newest=self.thread_top().childNewestMessageDate()
        return newest

class ZDTopic(ZCatalog.ZCatalog):
    """ZDTopic object"""

    meta_type = 'Z Discussions Topic Base Class'

    manage_options=(
        {'label': 'Contents',
	 'action': 'manage_main',
	 'target': 'manage_main'},
        
	{'label': 'Cataloged Objects',
	 'action': 'manage_catalogView',
	 'target': 'manage_main'},
        
	{'label': 'MetaData Table',
	 'action': 'manage_catalogSchema',
	 'target':'manage_main'},
        
	{'label': 'Indexes',
	 'action': 'manage_catalogIndexes',
	 'target':'manage_main'},
        
	{'label': 'Status',
	 'action': 'manage_catalogStatus', 
	 'target':'manage_main'},

        {'label': 'Advanced',
         'action': 'manage_catalogAdvanced',
         'target':'manage_main'},

        {'label': 'Undo',
         'action': 'manage_UndoForm',
         'target':'manage_main'}
        )

    def __init__(self, id, title=''):


      # Initialise ZCatalog

         ZCatalog.ZCatalog.__init__(self, id, title)
         self._initIndexes()

      # Set up the indexes

    def _initIndexes(self):
         """
         Check which version of Zope we're running under
         and setup the indexes for the catalog.
         """
         base = aq_base(self)
         if hasattr(base, 'addIndex'):
             # Zope 2.4
             addIndex = self.addIndex
         else:
             # Zope 2.3 and below
             addIndex = self._catalog.addIndex
         if hasattr(base, 'addColumn'):
             # Zope 2.4
             addColumn = self.addColumn
         else:
             # Zope 2.3 and below
             addColumn = self._catalog.addColumn
         for index_name, index_type in self.enumerateIndexes():
             addIndex( index_name, index_type )
         for column_name in self.enumerateColumns():
             addColumn( column_name )

    def enumerateIndexes(self):
         """
         Build Catalog Indexes from this list
         """
         return ( ('body', 'TextIndex')
                , ('author', 'FieldIndex')
                , ('email', 'FieldIndex')
                , ('in_reply_to', 'FieldIndex')
                , ('created', 'FieldIndex')
                , ('modified', 'FieldIndex')
                )

    def enumerateColumns(self):
         """
         Build Catalog columns from this list
         """
         return ('body'
               , 'author'
               , 'email'
               , 'in_reply_to'
               , 'created'
               , 'modified'
                )

    def manage_editZDTopic(self, title, REQUEST=None):
	"""Set properties of ZDTopic.  Presently just title."""

	self.title = title
	if REQUEST:
	    return self.manage_main(self, REQUEST, manage_tabs_message = "ZDTopic changed.")

    def posts(self, REQUEST=None):
	"""Return the top-level posts"""
	if REQUEST is None:
	    REQUEST=self.REQUEST
	posts = []
	for res in self.searchResults(in_reply_to=''): #, sort_on='created'):
	    posts.append(self.getobject(res.data_record_id_, REQUEST))
	return posts

    children = posts

    def generate_id(self):
	"""Find an available ID"""
	id = 0
	fmt = "%08d"
	while hasattr(self, fmt % id):
	    id = id + 1
	return fmt % id

    def manage_addZDItem(self, id, title, body, author, email, in_reply_to='', REQUEST=None):
	"""Add a ZDItem to this ZDTopic"""
    
	item = ZDItem(id, title, body, author, email, in_reply_to)
	self._setObject(id, item)
	if REQUEST is not None:
	    return self.manage_main(self, REQUEST)














